#!perl -w
#
# BlockSpecialFileTab -- block special files found in /dev
#   Copyright (C) 2005  Erik van Konijnenburg
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# This provides access to a table of descriptors, each containing
# path and devno.
#
# findByDevno -- find a descriptor with devno (there may be others)
# all -- list of all descriptors (may have duplicate devnos)
#
use strict;
use warnings;
use Conf;
use BlockSpecialFile;
package BlockSpecialFileTab;


my $bsfTab = undef;
my $bsfList = undef;


sub walkBlockSpecialFiles ($);

sub init () {
	if (defined ($bsfTab)) {
		return;
	}
	$bsfTab = {};
	my $devDir = Conf::get('dev');
	walkBlockSpecialFiles ($devDir);
	$bsfList = [ values %{$bsfTab} ];
}

sub walkBlockSpecialFiles ($) {
	my ($dirName) = @_;
	my $dir;
	if (! opendir ($dir, $dirName)) {
		Base::fatal ("can't open directory $dirName");
	}
	while (defined(my $entry = readdir($dir))) {
		next if $entry eq ".";
		next if $entry eq "..";
		my $path = "$dirName/$entry";
		if (-l $path) {
			next;
		}
		elsif (-b _) {
			my $devno = Base::devno ($path);

			# this may overwrite earlier entries;
			# unimportant since both pathnames
			# access the same device.
			$bsfTab->{$devno} = BlockSpecialFile->new(
				path => $path,
				devno => $devno);
		}
		elsif (-d _) {
			walkBlockSpecialFiles ($path);
		}
	}
	if (! closedir ($dir)) {
		Base::fatal ("could not read directory $dirName");
	}
}


#
# findByDevno -- return a working path given major:minor or undef.
# Note that there may be other files accessing the same device.
#
sub findByDevno ($) {
	my ($devno) = @_;
	init;
	return $bsfTab->{$devno};
}

#
# all -- return list of all descriptors.
#
sub all	() {
	init;
	return $bsfList;
}


1;
